/*global console,_ */
'use strict';

var oMIDIModule = angular.module('MIDI', []);

oMIDIModule.service('MIDIService', ['$rootScope', '$log', '$timeout', function($rootScope, $log, $timeout) {
	
	var oService = this;	
	var midi = null;  // global MIDIAccess object
	var outputId;
	var inputId;
	
	var receiveHandler;
	
	var outputEnabled = true;
	var debugEnabled;
	var throttleDelay;

	this.setReceiveHandler = function(aReceiveHandler) {
		receiveHandler = aReceiveHandler;
	};
	

	function listInputsAndOutputs() {
		if (!debugEnabled) {
			return;
		}
		var oInputs = midi.inputs;		
		oInputs.forEach(function(aInput) {
			$log.debug('Input id: '+aInput.id+' name: '+aInput.name);
		});

		midi.outputs.forEach(function(aOutput) {
			$log.debug('Output id: '+aOutput.id+' name: '+aOutput.name);
		});
	}

	function onMIDIMessage( event ) {
	  var str = "MIDI message received at timestamp " + event.timestamp + "[" + event.data.length + " bytes]: ";
	  for (var i=0; i<event.data.length; i++) {
		str += "0x" + event.data[i].toString(16) + " ";
	  }
	  $log.debug( str );
	}

	function startLoggingMIDIInput( aPortId ) {
		var oInput = midi.inputs.get(aPortId);
		oInput.onmidimessage = onMIDIMessage;
	}

	function onMIDISuccess( aMidiAccess ) {
		midi = aMidiAccess;  // store in the global (in real usage, would probably keep in an object instance)

		aMidiAccess.onstatechange = function (e) {
			if (debugEnabled) {
				$log.debug(e.port.name + ' appears!');
			}
			//listInputsAndOutputs();
			oService.selectInput(oService.getInputId());
			oService.selectOutput(oService.getOutputId());
		};
		
		listInputsAndOutputs();

		oService.selectInput(oService.getInputId());
		oService.selectOutput(oService.getOutputId());

	//	startLoggingMIDIInput('4');
		if (debugEnabled) {
			$log.debug( "MIDI ready" );
		}
		
		$rootScope.$broadcast('midi-ready');
	}

	function onMIDIFailure(msg) {
		alert( "Failed to get MIDI access - " + msg );
	}

	function getOutput() {
		var oOutput;
		if (outputEnabled && midi !== null) {
			oOutput = midi.outputs.get(oService.getOutputId()); 
		}
		return oOutput;
	}
	
	function getInput() {
		if (midi === null) {
			return undefined;
		}
		return midi.inputs.get(oService.getInputId());
	}
	
	this.init = function() {
		if (!navigator.requestMIDIAccess) {
			alert('No MIDI service available. Try Chrome or Opera.');
			return;
		}
		navigator.requestMIDIAccess({ sysex: true }).then( onMIDISuccess, onMIDIFailure );		
	};
	
	this.selectOutput = function(aOutputKey) {
		outputId = aOutputKey;
		if (localStorage) {
			localStorage.setItem('com.dqueffeulou.ObieEditor.midi.output.id', outputId);
		}
	};
	
	this.selectInput = function(aInputKey) {
		inputId = aInputKey;
		var oInput = getInput();
		if (oInput) {
			if (localStorage) {
				localStorage.setItem('com.dqueffeulou.ObieEditor.midi.input.id', inputId);
			}
			oInput.onmidimessage = function(aMidiMessageEvent) {
				if (debugEnabled) {
					$log.debug('receive MIDI message: '+aMidiMessageEvent);
				}
				if (receiveHandler) {
					receiveHandler(aMidiMessageEvent.data);
				}
			};
		}
	};
	
	this.sendData = function(aBytesArray) {
		var oOutput = getOutput();
		if (oOutput) {
			if (debugEnabled) {
				$log.debug('send: '+aBytesArray);
			}
			oOutput.send(aBytesArray);
		}
	};
	
	this.getThrottleDelay = function() {
		if (!throttleDelay && localStorage) {
			throttleDelay = parseInt(localStorage.getItem('com.dqueffeulou.ObieEditor.midi.throttle'));
		}
		return throttleDelay ? throttleDelay : 50;
	};
	
	this.setThrottleDelay = function(aThrottleDelay) {
		throttleDelay = aThrottleDelay;
		if (localStorage) {
			localStorage.setItem('com.dqueffeulou.ObieEditor.midi.throttle', aThrottleDelay.toString());
		}
	};
	
			
	this.getInputs = function() {
		var oInputs = [];
		if (midi) {
			midi.inputs.forEach(function(aInput) {
				oInputs.push(aInput);
			});
		}
		return oInputs;
	};
	
	this.getInputId = function() {
		if (!inputId) {
			if (localStorage) {
				inputId = localStorage.getItem('com.dqueffeulou.ObieEditor.midi.input.id');
			}
		}
		return inputId;
	};
	
	this.getOutputs = function() {
		var oOutputs = [];
		if (midi) {
			midi.outputs.forEach(function(aOutput) {
				oOutputs.push(aOutput);
			});
		}
		return oOutputs;
	};
	
	this.getOutputId = function() {
		if (!outputId) {
			if (localStorage) {
				outputId = localStorage.getItem('com.dqueffeulou.ObieEditor.midi.output.id');
			}
		}
		return outputId;
	};
	
	this.setOutputEnabled = function(aBool) {
		outputEnabled = aBool;
	};
	
	this.setDebugEnabled = function(aBool) {
		debugEnabled = aBool;
		if (localStorage) {
			localStorage.setItem('com.dqueffeulou.ObieEditor.midi.debug', aBool ? 'true' : 'false');
		}
	};
	
	this.getDebugEnabled = function() {
		if (!debugEnabled && localStorage) {
			debugEnabled = localStorage.getItem('com.dqueffeulou.ObieEditor.midi.debug') === 'true' ? true : false;
		}
		return debugEnabled;
	};

	this.sendAllNotesOff = function() {
		var oBytes = [0xB0, 0x7B, 0];
		this.sendData(oBytes);
	};
	

}]);

oMIDIModule.run(['MIDIService', function(MIDIService) {

	MIDIService.getDebugEnabled();
	MIDIService.init();	
}]);
